import 'package:c007_page_view_sync_list_view/helpers/colored_box_item.dart';
import 'package:c007_page_view_sync_list_view/helpers/rainbow_colors.dart';
import 'package:flutter/material.dart';
import 'package:scroll_to_index/scroll_to_index.dart';

class ListViewScreen extends StatefulWidget {
  const ListViewScreen({
    super.key,
    required this.allColors,
    required this.currentPageIndexNotifier,
  });
  final List<ColorInfo> allColors;
  final ValueNotifier<int> currentPageIndexNotifier;

  @override
  State<ListViewScreen> createState() => _ListViewScreenState();
}

class _ListViewScreenState extends State<ListViewScreen> {
  final _scrollController = AutoScrollController(
    suggestedRowHeight: 80, // approximate height of each list item
  );

  @override
  void initState() {
    super.initState();
    // * Add listener to sync ListView with index changes from PageView
    widget.currentPageIndexNotifier.addListener(_scrollToCurrentIndex);
  }

  @override
  void dispose() {
    widget.currentPageIndexNotifier.removeListener(_scrollToCurrentIndex);
    _scrollController.dispose();
    super.dispose();
  }

  void _scrollToCurrentIndex() {
    final index = widget.currentPageIndexNotifier.value;
    _scrollController.scrollToIndex(
      index,
      duration: const Duration(milliseconds: 300),
      preferPosition: AutoScrollPosition.middle,
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: ListView.builder(
        controller: _scrollController,
        itemCount: widget.allColors.length,
        itemBuilder: (_, index) {
          final colorInfo = widget.allColors[index];
          return ValueListenableBuilder<int>(
            valueListenable: widget.currentPageIndexNotifier,
            builder: (_, currentPageIndex, __) => AutoScrollTag(
              key: ValueKey(index),
              controller: _scrollController,
              index: index,
              child: GestureDetector(
                // * Update the notifier index when an item is selected
                onTap: () => widget.currentPageIndexNotifier.value = index,
                child: ColoredBoxItem(
                  colorInfo: colorInfo,
                  isSelected: index == currentPageIndex,
                ),
              ),
            ),
          );
        },
      ),
    );
  }
}
