import 'package:c007_page_view_sync_list_view/helpers/colored_box_item.dart';
import 'package:c007_page_view_sync_list_view/helpers/rainbow_colors.dart';
import 'package:flutter/material.dart';

class PageViewScreen extends StatefulWidget {
  const PageViewScreen({
    super.key,
    required this.allColors,
    required this.currentPageIndexNotifier,
  });
  final List<ColorInfo> allColors;
  final ValueNotifier<int> currentPageIndexNotifier;

  @override
  State<PageViewScreen> createState() => _PageViewScreenState();
}

class _PageViewScreenState extends State<PageViewScreen> {
  final _pageController = PageController();

  @override
  void initState() {
    super.initState();
    // * Add listener to sync PageView with index changes from ListView
    widget.currentPageIndexNotifier.addListener(_onPageIndexChanged);
  }

  @override
  void dispose() {
    widget.currentPageIndexNotifier.removeListener(_onPageIndexChanged);
    _pageController.dispose();
    super.dispose();
  }

  void _onPageIndexChanged() {
    if (_pageController.page?.round() !=
        widget.currentPageIndexNotifier.value) {
      _pageController.jumpToPage(widget.currentPageIndexNotifier.value);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: PageView(
        controller: _pageController,
        // * Update the notifier index when the page changes
        onPageChanged: (index) => widget.currentPageIndexNotifier.value = index,
        children: widget.allColors
            .map((color) => ColoredBoxItem(colorInfo: color))
            .toList(),
      ),
    );
  }
}
